@@ -1,8 +1,9 @@
 # -*- coding: latin-1 -*-
 #
-from pyshacl.validate import validate, Validator
+from pyshacl.validate import Validator, validate
+
 
 # version compliant with https://www.python.org/dev/peps/pep-0440/
-__version__ = '0.11.6.post1'
+__version__ = '0.12.1.post1'
 
 __all__ = ['validate', 'Validator', '__version__']
