@@ -1,24 +1,48 @@
 # -*- coding: utf-8 -*-
-from sys import stderr
+#
 import logging
-import rdflib
-import owlrl
 
+from functools import wraps
+from sys import stderr
+from typing import List, Optional, Tuple, Union
+
+import owlrl
+import rdflib
+
+from pyshacl.pytypes import GraphLike
 from pyshacl.rdfutil.clone import mix_datasets
 
+
 if owlrl.json_ld_available:
-    import rdflib_jsonld
-from rdflib import Literal, URIRef, BNode
+    import rdflib_jsonld  # noqa: F401
+
+from rdflib import BNode, Literal, URIRef
+
+from pyshacl.consts import (
+    RDF_object,
+    RDF_predicate,
+    RDF_subject,
+    RDF_type,
+    RDFS_Resource,
+    SH_conforms,
+    SH_result,
+    SH_resultMessage,
+    SH_ValidationReport,
+)
 from pyshacl.errors import ReportableRuntimeError, ValidationFailure
-from pyshacl.inference import CustomRDFSSemantics, CustomRDFSOWLRLSemantics
-from pyshacl.shapes_graph import ShapesGraph
-from pyshacl.consts import RDF_type, SH_conforms, \
-    SH_result, SH_ValidationReport, RDFS_Resource, SH_resultMessage, \
-    SH_sourceShape, SH_sourceConstraint, SH_resultPath, RDF_object, RDF_subject, RDF_predicate
-from pyshacl.rules import gather_rules, apply_rules, gather_functions
-from pyshacl.rdfutil import load_from_source, clone_graph, \
-    clone_node, compare_blank_node, mix_graphs, order_graph_literal
+from pyshacl.inference import CustomRDFSOWLRLSemantics, CustomRDFSSemantics
 from pyshacl.monkey import apply_patches, rdflib_bool_patch, rdflib_bool_unpatch
+from pyshacl.rdfutil import (
+    clone_graph,
+    clone_node,
+    compare_blank_node,
+    load_from_source,
+    mix_graphs,
+    order_graph_literal,
+)
+from pyshacl.rules import apply_rules, gather_functions, gather_rules
+from pyshacl.shapes_graph import ShapesGraph
+
 
 log_handler = logging.StreamHandler(stderr)
 log = logging.getLogger(__name__)
@@ -31,7 +55,7 @@ log_handler.setLevel(logging.INFO)
 
 class Validator(object):
     @classmethod
-    def _load_default_options(cls, options_dict):
+    def _load_default_options(cls, options_dict: dict):
         options_dict.setdefault('advanced', False)
         options_dict.setdefault('inference', 'none')
         options_dict.setdefault('abort_on_error', False)
@@ -39,7 +63,9 @@ class Validator(object):
             options_dict['logger'] = logging.getLogger(__name__)
 
     @classmethod
-    def _run_pre_inference(cls, target_graph, inference_option, logger=None):
+    def _run_pre_inference(
+        cls, target_graph: GraphLike, inference_option: str, logger: Optional[logging.Logger] = None
+    ):
         """
         Note, this is the OWL/RDFS pre-inference,
         it is not the Advanced Spec SHACL-Rule inferencing step.
@@ -54,23 +80,23 @@ class Validator(object):
                 inferencer = owlrl.DeductiveClosure(CustomRDFSSemantics)
             elif inference_option == 'owlrl':
                 inferencer = owlrl.DeductiveClosure(owlrl.OWLRL_Semantics)
-            elif inference_option == 'both' or inference_option == 'all'\
-                    or inference_option == 'rdfsowlrl':
+            elif inference_option == 'both' or inference_option == 'all' or inference_option == 'rdfsowlrl':
                 inferencer = owlrl.DeductiveClosure(CustomRDFSOWLRLSemantics)
             else:
-                raise ReportableRuntimeError(
-                    "Don't know how to do '{}' type inferencing."
-                    .format(inference_option))
+                raise ReportableRuntimeError("Don't know how to do '{}' type inferencing.".format(inference_option))
         except Exception as e:  # pragma: no cover
             logger.error("Error during creation of OWL-RL Deductive Closure")
             if isinstance(e, ReportableRuntimeError):
                 raise e
-            raise ReportableRuntimeError("Error during creation of OWL-RL Deductive Closure\n"
-                                         "{}".format(str(e.args[0])))
+            raise ReportableRuntimeError(
+                "Error during creation of OWL-RL Deductive Closure\n" "{}".format(str(e.args[0]))
+            )
         if isinstance(target_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph)):
             named_graphs = [
                 rdflib.Graph(target_graph.store, i, namespace_manager=target_graph.namespace_manager)
-                if not isinstance(i, rdflib.Graph) else i for i in target_graph.store.contexts(None)
+                if not isinstance(i, rdflib.Graph)
+                else i
+                for i in target_graph.store.contexts(None)
             ]
         else:
             named_graphs = [target_graph]
@@ -79,22 +105,19 @@ class Validator(object):
                 inferencer.expand(g)
         except Exception as e:  # pragma: no cover
             logger.error("Error while running OWL-RL Deductive Closure")
-            raise ReportableRuntimeError("Error while running OWL-RL Deductive Closure\n"
-                                         "{}".format(str(e.args[0])))
+            raise ReportableRuntimeError("Error while running OWL-RL Deductive Closure\n" "{}".format(str(e.args[0])))
 
     @classmethod
-    def create_validation_report(cls, conforms, target_graph, shacl_graph, results):
+    def create_validation_report(cls, conforms: bool, results: List[Tuple]):
         v_text = "Validation Report\nConforms: {}\n".format(str(conforms))
         result_len = len(results)
         if not conforms:
-            assert result_len > 0, \
-                "A Non-Conformant Validation Report must have at least one result."
+            assert result_len > 0, "A Non-Conformant Validation Report must have at least one result."
         if result_len > 0:
             v_text += "Results ({}):\n".format(str(result_len))
         vg = rdflib.Graph()
-        sg = shacl_graph.graph
-        for p, n in sg.namespace_manager.namespaces():
-            vg.namespace_manager.bind(p, n)
+        # for p, n in sg.namespace_manager.namespaces():
+        #     vg.namespace_manager.bind(p, n)
         vr = BNode()
         vg.add((vr, RDF_type, SH_ValidationReport))
         vg.add((vr, SH_conforms, Literal(conforms)))
@@ -107,36 +130,36 @@ class Validator(object):
                 if isinstance(o, tuple):
                     source = o[0]
                     node = o[1]
-                    if source == 'S':
-                        o = clone_node(sg, node, vg)
-                    elif source == 'D':
-                        o = clone_node(target_graph, node, vg)
-                    else:  # pragma: no cover
-                        raise RuntimeError("Adding node to validation report must have source of either 'D' or 'S'.")
+                    o = clone_node(source, node, vg)
                 vg.add((s, p, o))
         return vg, v_text
 
-    def __init__(self, data_graph, *args, shacl_graph=None,
-                 ont_graph=None, options=None, **kwargs):
+    def __init__(
+        self,
+        data_graph: GraphLike,
+        *args,
+        shacl_graph: Optional[GraphLike] = None,
+        ont_graph: Optional[GraphLike] = None,
+        options: Optional[dict] = None,
+        **kwargs,
+    ):
         options = options or {}
         self._load_default_options(options)
-        self.options = options
-        self.logger = options['logger']
+        self.options = options  # type: dict
+        self.logger = options['logger']  # type: logging.Logger
         self.pre_inferenced = kwargs.pop('pre_inferenced', False)
-        assert isinstance(data_graph, rdflib.Graph),\
-            "data_graph must be a rdflib Graph object"
+        if not isinstance(data_graph, rdflib.Graph):
+            raise RuntimeError("data_graph must be a rdflib Graph object")
         self.data_graph = data_graph
         self._target_graph = None
         self.ont_graph = ont_graph
         self.data_graph_is_multigraph = isinstance(self.data_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph))
-        if self.ont_graph is not None and \
-            isinstance(self.ont_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph)):
+        if self.ont_graph is not None and isinstance(self.ont_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph)):
             self.ont_graph.default_union = True
 
         if shacl_graph is None:
             shacl_graph = clone_graph(data_graph, identifier='shacl')
-        assert isinstance(shacl_graph, rdflib.Graph),\
-            "shacl_graph must be a rdflib Graph object"
+        assert isinstance(shacl_graph, rdflib.Graph), "shacl_graph must be a rdflib Graph object"
         self.shacl_graph = ShapesGraph(shacl_graph, self.logger)
 
     @property
@@ -162,14 +185,10 @@ class Validator(object):
                 self._run_pre_inference(the_target_graph, inference_option, self.logger)
                 self.pre_inferenced = True
         self._target_graph = the_target_graph
-        reports = []
-        non_conformant = False
+
         shapes = self.shacl_graph.shapes  # This property getter triggers shapes harvest.
         if self.options['advanced']:
-            advanced = {
-                'functions': gather_functions(self.shacl_graph),
-                'rules': gather_rules(self.shacl_graph)
-            }
+            advanced = {'functions': gather_functions(self.shacl_graph), 'rules': gather_rules(self.shacl_graph)}
             for s in shapes:
                 s.set_advanced(True)
         else:
@@ -177,59 +196,89 @@ class Validator(object):
         if isinstance(the_target_graph, (rdflib.Dataset, rdflib.ConjunctiveGraph)):
             named_graphs = [
                 rdflib.Graph(the_target_graph.store, i, namespace_manager=the_target_graph.namespace_manager)
-                if not isinstance(i, rdflib.Graph) else i for i in the_target_graph.store.contexts(None)
+                if not isinstance(i, rdflib.Graph)
+                else i
+                for i in the_target_graph.store.contexts(None)
             ]
         else:
             named_graphs = [the_target_graph]
+        reports = []
+        non_conformant = False
         for g in named_graphs:
             if advanced:
-                #apply functions?
+                # TODO: apply functions?
                 apply_rules(advanced['rules'], g)
             for s in shapes:
                 _is_conform, _reports = s.validate(g)
                 non_conformant = non_conformant or (not _is_conform)
                 reports.extend(_reports)
-        v_report, v_text = self.create_validation_report(
-            not non_conformant, the_target_graph, self.shacl_graph, reports)
+        v_report, v_text = self.create_validation_report(not non_conformant, reports)
         return (not non_conformant), v_report, v_text
 
 
-def meta_validate(shacl_graph, inference='rdfs', **kwargs):
-    shacl_shacl_graph = meta_validate.shacl_shacl_graph
-    if shacl_shacl_graph is None:
-        from os import path
-        import pickle
-        import sys
-        if getattr( sys, 'frozen', False ) :
+def with_metashacl_shacl_graph_cache(f):
+    # noinspection PyPep8Naming
+    EMPTY = object()
+
+    @wraps(f)
+    def wrapped(*args, **kwargs):
+        graph_cache = getattr(wrapped, "graph_cache", None)
+        assert graph_cache is not None
+        if graph_cache is EMPTY:
+            import pickle
+            import sys
+
+            from os import path
+
+            if getattr(sys, 'frozen', False):
                 # runs in a pyinstaller bundle
                 here_dir = sys._MEIPASS
                 pickle_file = path.join(here_dir, "shacl-shacl.pickle")
-        else :
+            else:
                 here_dir = path.dirname(__file__)
                 pickle_file = path.join(here_dir, "shacl-shacl.pickle")
-        with open(pickle_file, 'rb') as shacl_pickle:
-            u = pickle.Unpickler(shacl_pickle, fix_imports=False)
-            shacl_shacl_store = u.load()
-        shacl_shacl_graph = rdflib.Graph(store=shacl_shacl_store, identifier="http://www.w3.org/ns/shacl-shacl")
-        meta_validate.shacl_shacl_graph = shacl_shacl_graph
-    shacl_graph = load_from_source(shacl_graph, rdf_format=kwargs.pop('shacl_graph_format', None),
-                                   multigraph=True)
+            with open(pickle_file, 'rb') as shacl_pickle:
+                u = pickle.Unpickler(shacl_pickle, fix_imports=False)
+                shacl_shacl_store = u.load()
+            shacl_shacl_graph = rdflib.Graph(store=shacl_shacl_store, identifier="http://www.w3.org/ns/shacl-shacl")
+            setattr(wrapped, "graph_cache", shacl_shacl_graph)
+        return f(*args, **kwargs)
+
+    setattr(wrapped, "graph_cache", EMPTY)
+    return wrapped
+
+
+@with_metashacl_shacl_graph_cache
+def meta_validate(shacl_graph: Union[GraphLike, str], inference: Optional[str] = 'rdfs', **kwargs):
+    shacl_shacl_graph = meta_validate.graph_cache
+    shacl_graph = load_from_source(shacl_graph, rdf_format=kwargs.pop('shacl_graph_format', None), multigraph=True)
     _ = kwargs.pop('meta_shacl', None)
     return validate(shacl_graph, shacl_graph=shacl_shacl_graph, inference=inference, **kwargs)
-meta_validate.shacl_shacl_graph = None
 
 
-def validate(data_graph, *args, shacl_graph=None, ont_graph=None, advanced=False, inference=None, abort_on_error=False, **kwargs):
+def validate(
+    data_graph: Union[GraphLike, str, bytes],
+    *args,
+    shacl_graph: Optional[Union[GraphLike, str, bytes]] = None,
+    ont_graph: Optional[Union[GraphLike, str, bytes]] = None,
+    advanced: Optional[bool] = False,
+    inference: Optional[str] = None,
+    abort_on_error: Optional[bool] = False,
+    **kwargs,
+):
     """
     :param data_graph: rdflib.Graph or file path or web url of the data to validate
-    :type data_graph: rdflib.Graph | str
+    :type data_graph: rdflib.Graph | str | bytes
     :param args:
     :type args: list
-    :param shacl_graph: rdflib.Graph or file path or web url of the SHACL Shapes graph to use to validate the data graph
-    :type shacl_graph: rdflib.Graph | str
+    :param shacl_graph: rdflib.Graph or file path or web url of the SHACL Shapes graph to use to
+    validate the data graph
+    :type shacl_graph: rdflib.Graph | str | bytes
     :param ont_graph: rdflib.Graph or file path or web url of an extra ontology document to mix into the data graph
-    :type ont_graph: rdflib.Graph | str
-    :param inference:
+    :type ont_graph: rdflib.Graph | str | bytes
+    :param advanced: Enable advanced SHACL features, default=False
+    :type advanced: bool | None
+    :param inference: One of "rdfs", "owlrl", "both", "none", or None
     :type inference: str | None
     :param abort_on_error:
     :type abort_on_error: bool | None
@@ -241,40 +290,39 @@ def validate(data_graph, *args, shacl_graph=None, ont_graph=None, advanced=False
         log_handler.setLevel(logging.DEBUG)
         log.setLevel(logging.DEBUG)
     apply_patches()
-    do_check_dash_result = kwargs.pop('check_dash_result', False)
-    do_check_sht_result = kwargs.pop('check_sht_result', False)
+    do_check_dash_result = kwargs.pop('check_dash_result', False)  # type: bool
+    do_check_sht_result = kwargs.pop('check_sht_result', False)  # type: bool
     if kwargs.get('meta_shacl', False):
         to_meta_val = shacl_graph or data_graph
         conforms, v_r, v_t = meta_validate(to_meta_val, inference=inference, **kwargs)
         if not conforms:
-            msg = "Shacl File does not validate against the Shacl Shapes Shacl file.\n{}"\
-                  .format(v_t)
+            msg = "Shacl File does not validate against the Shacl Shapes Shacl file.\n{}".format(v_t)
             log.error(msg)
             raise ReportableRuntimeError(msg)
     do_owl_imports = kwargs.pop('do_owl_imports', False)
     data_graph_format = kwargs.pop('data_graph_format', None)
-    data_graph = load_from_source(data_graph,
-                                  rdf_format=data_graph_format,
-                                  multigraph=True,
-                                  do_owl_imports=False)  # no imports on data_graph
+    # force no owl imports on data_graph
+    data_graph = load_from_source(data_graph, rdf_format=data_graph_format, multigraph=True, do_owl_imports=False)
     ont_graph_format = kwargs.pop('ont_graph_format', None)
     if ont_graph is not None:
-        ont_graph = load_from_source(ont_graph,
-                                     rdf_format=ont_graph_format,
-                                     multigraph=True,
-                                     do_owl_imports=do_owl_imports)
+        ont_graph = load_from_source(
+            ont_graph, rdf_format=ont_graph_format, multigraph=True, do_owl_imports=do_owl_imports
+        )
     shacl_graph_format = kwargs.pop('shacl_graph_format', None)
     if shacl_graph is not None:
         # SHACL spec requires rdf BOOL literals to operate in a very specific way
         rdflib_bool_patch()
-        shacl_graph = load_from_source(shacl_graph, rdf_format=shacl_graph_format, multigraph=True,
-                                       do_owl_imports=do_owl_imports)
+        shacl_graph = load_from_source(
+            shacl_graph, rdf_format=shacl_graph_format, multigraph=True, do_owl_imports=do_owl_imports
+        )
         rdflib_bool_unpatch()
     try:
         validator = Validator(
-            data_graph, shacl_graph=shacl_graph, ont_graph=ont_graph,
-            options={'inference': inference, 'abort_on_error': abort_on_error,
-                     'advanced': advanced, 'logger': log})
+            data_graph,
+            shacl_graph=shacl_graph,
+            ont_graph=ont_graph,
+            options={'inference': inference, 'abort_on_error': abort_on_error, 'advanced': advanced, 'logger': log},
+        )
         conforms, report_graph, report_text = validator.run()
     except ValidationFailure as e:
         conforms = False
@@ -285,14 +333,15 @@ def validate(data_graph, *args, shacl_graph=None, ont_graph=None, advanced=False
         return passes, report_graph, report_text
     if do_check_sht_result:
         (sht_graph, sht_result_node) = kwargs.pop('sht_validate', (False, None))
+        if not sht_result_node:
+            raise RuntimeError("Cannot check SHT result if SHT graph and result node are not given.")
         passes = check_sht_result(report_graph, sht_graph or shacl_graph or data_graph, sht_result_node)
         return passes, report_graph, report_text
     do_serialize_report_graph = kwargs.pop('serialize_report_graph', False)
     if do_serialize_report_graph and isinstance(report_graph, rdflib.Graph):
         if not (isinstance(do_serialize_report_graph, str)):
             do_serialize_report_graph = 'turtle'
-        report_graph = report_graph.serialize(None, encoding='utf-8',
-                                              format=do_serialize_report_graph)
+        report_graph = report_graph.serialize(None, encoding='utf-8', format=do_serialize_report_graph)
     return conforms, report_graph, report_text
 
 
@@ -331,7 +380,9 @@ def compare_validation_reports(report_graph, expected_graph, expected_result):
     expected_conforms = expected_graph.objects(expected_result, SH_conforms)
     expected_conforms = set(expected_conforms)
     if len(expected_conforms) < 1:  # pragma: no cover
-        raise ReportableRuntimeError("Cannot check the expected result, the given expectedResult does not have an sh:conforms.")
+        raise ReportableRuntimeError(
+            "Cannot check the expected result, the given expectedResult does not have an sh:conforms."
+        )
     expected_conforms = next(iter(expected_conforms))
     expected_result_nodes = expected_graph.objects(expected_result, SH_result)
     expected_result_nodes = set(expected_result_nodes)
@@ -340,17 +391,20 @@ def compare_validation_reports(report_graph, expected_graph, expected_result):
     validation_reports = report_graph.subjects(RDF_type, SH_ValidationReport)
     validation_reports = set(validation_reports)
     if len(validation_reports) < 1:  # pragma: no cover
-        raise ReportableRuntimeError("Cannot check the validation report, the report graph does not contain a ValidationReport")
+        raise ReportableRuntimeError(
+            "Cannot check the validation report, the report graph does not contain a ValidationReport"
+        )
     validation_report = next(iter(validation_reports))
-    clean_validation_reports(
-        report_graph, validation_report, expected_graph, expected_result)
+    clean_validation_reports(report_graph, validation_report, expected_graph, expected_result)
     eq = compare_blank_node(report_graph, validation_report, expected_graph, expected_result)
     if eq != 0:
         return False
     report_conforms = report_graph.objects(validation_report, SH_conforms)
     report_conforms = set(report_conforms)
     if len(report_conforms) < 1:  # pragma: no cover
-        raise ReportableRuntimeError("Cannot check the validation report, the report graph does not have an sh:conforms.")
+        raise ReportableRuntimeError(
+            "Cannot check the validation report, the report graph does not have an sh:conforms."
+        )
     report_conforms = next(iter(report_conforms))
 
     if bool(expected_conforms.value) != bool(report_conforms.value):
@@ -364,26 +418,32 @@ def compare_validation_reports(report_graph, expected_graph, expected_result):
 
     if expected_result_node_count != report_result_node_count:
         # TODO:coverage: write a test for this
-        log.error("Number of expected result's sh:result entries is different from Validation Report's sh:result entries.\n"
-                  "Expected {}, got {}.".format(expected_result_node_count, report_result_node_count))
+        log.error(
+            "Number of expected result's sh:result entries is different from Validation Report's sh:result entries.\n"
+            "Expected {}, got {}.".format(expected_result_node_count, report_result_node_count)
+        )
         return False
     return True
 
-def compare_inferencing_reports(data_graph, expected_graph, expected_result):
+
+def compare_inferencing_reports(data_graph: GraphLike, expected_graph: GraphLike, expected_result: GraphLike):
     expected_object = set(expected_graph.objects(expected_result, RDF_object))
     if len(expected_object) < 1:
         raise ReportableRuntimeError(
-            "Cannot check the expected result, the given expectedResult does not have an rdf:object.")
+            "Cannot check the expected result, the given expectedResult does not have an rdf:object."
+        )
     expected_object = next(iter(expected_object))
     expected_subject = set(expected_graph.objects(expected_result, RDF_subject))
     if len(expected_subject) < 1:
         raise ReportableRuntimeError(
-            "Cannot check the expected result, the given expectedResult does not have an rdf:subject.")
+            "Cannot check the expected result, the given expectedResult does not have an rdf:subject."
+        )
     expected_subject = next(iter(expected_subject))
     expected_predicate = set(expected_graph.objects(expected_result, RDF_predicate))
     if len(expected_predicate) < 1:
         raise ReportableRuntimeError(
-            "Cannot check the expected result, the given expectedResult does not have an rdf:predicate.")
+            "Cannot check the expected result, the given expectedResult does not have an rdf:predicate."
+        )
     expected_predicate = next(iter(expected_predicate))
     if isinstance(expected_object, Literal):
         found_objs = set(data_graph.objects(expected_subject, expected_predicate))
@@ -411,8 +471,7 @@ def compare_inferencing_reports(data_graph, expected_graph, expected_result):
     return True
 
 
-
-def check_dash_result(data_graph, report_graph, expected_result_graph):
+def check_dash_result(data_graph: GraphLike, report_graph: GraphLike, expected_result_graph: GraphLike):
     DASH = rdflib.namespace.Namespace('http://datashapes.org/dash#')
     DASH_GraphValidationTestCase = DASH.term('GraphValidationTestCase')
     DASH_InferencingTestCase = DASH.term('InferencingTestCase')
@@ -428,7 +487,9 @@ def check_dash_result(data_graph, report_graph, expected_result_graph):
         expected_results = expected_result_graph.objects(test_case, DASH_expectedResult)
         expected_results = set(expected_results)
         if len(expected_results) < 1:  # pragma: no cover
-            raise ReportableRuntimeError("Cannot check the expected result, the given GraphValidationTestCase does not have an expectedResult.")
+            raise ReportableRuntimeError(
+                "Cannot check the expected result, the given GraphValidationTestCase does not have an expectedResult."
+            )
         expected_result = next(iter(expected_results))
         gv_res = compare_validation_reports(report_graph, expected_result_graph, expected_result)
     else:
@@ -438,19 +499,24 @@ def check_dash_result(data_graph, report_graph, expected_result_graph):
         expected_results = expected_result_graph.objects(test_case, DASH_expectedResult)
         expected_results = set(expected_results)
         if len(expected_results) < 1:  # pragma: no cover
-            raise ReportableRuntimeError("Cannot check the expected result, the given InferencingTestCase does not have an expectedResult.")
+            raise ReportableRuntimeError(
+                "Cannot check the expected result, the given InferencingTestCase does not have an expectedResult."
+            )
         expected_result = next(iter(expected_results))
         inf_res = compare_inferencing_reports(data_graph, expected_result_graph, expected_result)
     else:
         inf_res = True
-    if gv_res is None and inf_res is None: # pragma: no cover
-        raise ReportableRuntimeError("Cannot check the expected result, the given expected result graph does not have a GraphValidationTestCase or InferencingTestCase.")
+    if gv_res is None and inf_res is None:  # pragma: no cover
+        raise ReportableRuntimeError(
+            "Cannot check the expected result, the given expected result graph does not have a GraphValidationTestCase or InferencingTestCase."
+        )
     return gv_res and inf_res
 
-def check_sht_result(report_graph, sht_graph, sht_result_node):
+
+def check_sht_result(report_graph: GraphLike, sht_graph: GraphLike, sht_result_node: Union[URIRef, BNode]):
     SHT = rdflib.namespace.Namespace('http://www.w3.org/ns/shacl-test#')
     types = set(sht_graph.objects(sht_result_node, RDF_type))
-    expected_failure = (sht_result_node == SHT.Failure)
+    expected_failure = sht_result_node == SHT.Failure
     if expected_failure and isinstance(report_graph, ValidationFailure):
         return True
     elif isinstance(report_graph, ValidationFailure):
@@ -459,14 +525,10 @@ def check_sht_result(report_graph, sht_graph, sht_result_node):
         return False
     elif expected_failure:
         # TODO:coverage: write a test for this
-        log.error("SHT expects a Validation Failure, but the Validation Report does not indicate a Validation Failure.")
+        log.error(
+            "SHT expects a Validation Failure, but the Validation Report does not indicate a Validation Failure."
+        )
         return False
     if SH_ValidationReport not in types:
-        raise ReportableRuntimeError(
-            "SHT expected result must have type sh:ValidationReport")
+        raise ReportableRuntimeError("SHT expected result must have type sh:ValidationReport")
     return compare_validation_reports(report_graph, sht_graph, sht_result_node)
-
-
-
-
-
